#include <jni.h>
#include <string>
#include <dlfcn.h>
#include <unistd.h>
#include <fcntl.h>
#include "dobby.h"
#include "LogUtil.h"
#include "xdl.h"




extern "C" JNIEXPORT jstring JNICALL
Java_com_xs_xsframework_MainActivity_stringFromJNI(
        JNIEnv* env,
        jobject /* this */) {
    std::string hello = "Hello from C++ module2";
    return env->NewStringUTF(hello.c_str());
}

JNINativeMethod nativeMethods[] {
        {"hello", "(I)V", (void *)"native_hello"},
        {"world", "(J)V", (void *)"native_world"}
};

size_t (*strlen_backup)(const char* const str1);
size_t fake_strlen(char* str1){
    return strlen_backup(str1);
}

typedef int (*hook_ptr_t)(void *address, void *replace_call, void **origin_call);
hook_ptr_t hook_func = nullptr;
static bool init(){
    void * eng = dlopen("libengcore.so", RTLD_NOW);
    if(eng == nullptr){
        LOGE("open engcore failed");
        return false;
    }
    hook_func = (hook_ptr_t)dlsym(eng, "db_hk_wrapper");
    if(hook_func == nullptr){
        LOGE("db_hk_wrapper init failed");
        return false;
    }

    return true;
}


static int (*open_backup)(const char *pathname, int flags, mode_t mode) = nullptr;
static int (*openat_backup)(int dirfd, const char *pathname, int flags, mode_t mode) = nullptr;
int fake_open(const char *pathname, int flags, mode_t mode) {
//    LOGI("call fake_open:%s", pathname);
    return open_backup(pathname, flags, mode);
}

int fake_openat(int dirfd, const char *pathname, int flags, mode_t mode) {
//    LOGI("call fake_openat:%s", pathname);
    return openat_backup(dirfd, pathname, flags, mode);
}

void hook_open() {
    typedef int (*open_func_t)(const char *, int, ...); // 定义函数指针类型
    int ret = DobbyHook((void *)(open_func_t)open, (void *)fake_open, (void **)&open_backup);
    if (ret != 0) {
        LOGI("hook_open failed");
    }

    // Hook openat
    // 获取 openat 的地址
    void *handle = dlopen("libc.so", RTLD_NOW);
    if (handle == nullptr) {
        LOGI("Failed to load libc.so: %s", dlerror());
        return;
    }
    void *openat_addr = dlsym(handle, "openat");
    if (openat_addr == nullptr) {
        LOGI("Failed to find symbol: openat\n");
        return;
    }
    LOGI("openat_addr:%p", openat_addr);
    ret = DobbyHook(openat_addr, (void *)fake_openat, (void **)&openat_backup);
    if (ret != 0) {
        LOGI("hook_openat failed");
    }
}

static int (* add_back)(void *, void *, int, int ) = nullptr;
int fake_add(void * env, void * thiz, int a, int b){
    int ret =  add_back(env, thiz, a, b);
    LOGI("call fake_add(%d:%d)=%d", a, b, ret);
    return ret;
}

static void* (*do_dlopen_backup)(const char* name, int flags,
                       const void * extinfo,
                       const void* caller_addr) = nullptr;
void* fake_do_dlopen(const char* name, int flags,
                const void * extinfo,
                const void* caller_addr){

    void * handle = do_dlopen_backup(name, flags, extinfo, caller_addr);
    if(strstr(name, "libtest.so")){
        LOGI("call fake_do_dlopen:%s", name);
        void* ptestso = xdl_open("libtest.so", XDL_DEFAULT);
        LOGI("ptestso=%p", ptestso);
        void * pAdd = xdl_sym(ptestso, "Java_com_armcloud_testapp_MainActivity_add", nullptr);
        LOGI("pAdd=%p", pAdd);
        if(pAdd){
            int ret = DobbyHook(pAdd, (void *)fake_add, (void **)&add_back);
            LOGI("hook add ret=%d", ret);
        }
    }
    return handle;
}

void hook_linker(){
    void* pLinker = xdl_open("linker64",XDL_DEFAULT);
    if(pLinker == nullptr){
        LOGE("dlopen linker64 failed");
        return;
    }

    void* pdlopen= xdl_sym(pLinker, "__dl__Z9do_dlopenPKciPK17android_dlextinfoPKv", nullptr);
    if(pdlopen == nullptr){
        pdlopen= xdl_dsym(pLinker, "__dl__Z9do_dlopenPKciPK17android_dlextinfoPKv", nullptr);
        if(pdlopen == nullptr){
            LOGE("query pdlopen failed");
            return;
        }
    }

    int ret = DobbyHook(pdlopen, (void *) fake_do_dlopen, (void **)&do_dlopen_backup);
    if (ret != 0) {
        LOGI("hook_linker failed");
    }

}
jint JNI_OnLoad(JavaVM* vm, void* reserved) {
    JNIEnv *env = NULL;
    vm->GetEnv((void **)&env, JNI_VERSION_1_6);

    LOGI("call JNI_OnLoad from module so");

//    int ret = DobbyHook((void *) strlen, (void *)fake_strlen, (void **)&strlen_backup);
//    LOGI("hook strlen ret=%d hook=%p", ret, hook_func);
//
//    if(strlen_backup == nullptr){
//        LOGI("strlen_backup == null hook failed");
//    }

    hook_open();
    hook_linker();
//    LOGI("DobbyBuildVersion():%s", DobbyBuildVersion());
    return JNI_VERSION_1_6;
}