package androidx.app;

import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.os.Process;
import android.util.Log;
import android.view.View;
import com.android.core.XC_MethodHook;
import com.android.core.XSBridge;
import com.android.core.XSHelpers;


public class Entry {

    private static final String TAG = "___HKAPP___";
    private static Context globalContext;

    public static void appMain(ClassLoader loader, Context context, String appClass, String pkg, String process) {
        Log.d(TAG, "appClass=" + appClass + ", filesDir=" + context.getFilesDir() + ", pid=" + Process.myPid() + ", process=" + process);
        if(process.contains("sandboxed_process")){
            return;//webview相关的暂时忽略
        }
        if(pkg.equals("com.armcloud.testapp")) {
            try {
                Log.d(TAG, "do hookTestApp");
                Class<?> MainActivity = loader.loadClass("com.armcloud.testapp.MainActivity");
                XSHelpers.findAndHookMethod(MainActivity, "stringFromJNI", String.class, new XC_MethodHook(){
                    @Override
                    protected void beforeHookedMethod(MethodHookParam param) throws Throwable {
                        super.beforeHookedMethod(param);
                        Log.d(TAG, "call MainActivity->stringFromJNI arg=" + param.args[0]);
                    }

                    @Override
                    protected void afterHookedMethod(MethodHookParam param) throws Throwable {
                        super.afterHookedMethod(param);
                        Log.d(TAG, "call stringFromJNI return:" + param.getResult());
                        param.setResult("byebye");
                    }
                });

                // android.app.Activity
                XSHelpers.findAndHookMethod(Activity.class, "onCreate", Bundle.class, new XC_MethodHook(){
                    @Override
                    protected void beforeHookedMethod(MethodHookParam param) throws Throwable {
                        super.beforeHookedMethod(param);
                        Log.d(TAG, "call MainActivity->onCreate:" + param.args[0]);
                    }
                });
                Log.d(TAG, "exit hookTestApp");
            } catch (Exception e) {
                Log.d(TAG, "hookTestApp failed:\n" + Log.getStackTraceString(e));
            }
        }
//        try {
//            doJavaHook(pkg, loader);
////            doNativeHook();
//        }catch (Exception ex){
//            Log.d(TAG, Log.getStackTraceString(ex));
//        }
    }


    private static void doNativeHook(){
        Log.d(TAG, "load module2.so");
        System.loadLibrary("module2");
        Log.d(TAG, "load module2.so success");
    }

    private static void doJavaHook(String pkg, ClassLoader loader) {
        if(pkg.equals("com.armcloud.testapp")){
            hookTestApp(loader);
        }else if(pkg.equals("com.liuzh.deviceinfo")){
            hookDeviceInfoApp(loader);
        }else if(pkg.equals("com.coolapk.market")){
            hookCoolAn(loader);
        }else if(pkg.equals("com.zhiliaoapp.musically")){
            hookTiktok(loader);
        }else if(pkg.equals("com.whatsapp")){
            WhatsApp.doSomething();
        }
    }

    private static void hookTiktok(ClassLoader loader) {
        try {
            //com.ss.android.ugc.aweme.app.host.AwemeHostApplication
            Class<?> AwemeHostApplication = loader.loadClass("com.ss.android.ugc.aweme.app.host.AwemeHostApplication");
            XSHelpers.findAndHookMethod(AwemeHostApplication, "attachBaseContext", Context.class, new XC_MethodHook(){
                @Override
                protected void beforeHookedMethod(MethodHookParam param) throws Throwable {
                    super.beforeHookedMethod(param);
                    Log.d(TAG, "call AwemeHostApplication->attachBaseContext");
                }
            });

            XSHelpers.findAndHookMethod(AwemeHostApplication, "onCreate", new XC_MethodHook(){
                @Override
                protected void beforeHookedMethod(MethodHookParam param) throws Throwable {
                    super.beforeHookedMethod(param);
                    Log.d(TAG, "call AwemeHostApplication->onCreate");
                }
            });

        }catch (Exception ex){
            Log.d(TAG, "hookTiktok failed:\n" + Log.getStackTraceString(ex));
        }
    }

    private static void hookCoolAn(ClassLoader loader) {
        try {
            Class<?> MyApplication = loader.loadClass("com.netease.nis.wrapper.MyApplication");
            XSHelpers.findAndHookMethod(MyApplication, "attachBaseContext", Context.class, new XC_MethodHook(){
                @Override
                protected void beforeHookedMethod(MethodHookParam param) throws Throwable {
                    super.beforeHookedMethod(param);
                    Log.d(TAG, "call MyApplication->attachBaseContext");
                }
            });
        }catch (Exception ex){
            Log.d(TAG, "hookCoolAn failed:\n" + Log.getStackTraceString(ex));
        }
    }

    private static void hookTestApp(ClassLoader loader){
        try {
            Log.d(TAG, "do hookTestApp");
            Class<?> MainActivity = loader.loadClass("com.armcloud.testapp.MainActivity");
            XSHelpers.findAndHookMethod(MainActivity, "stringFromJNI", String.class, new XC_MethodHook(){
                @Override
                protected void beforeHookedMethod(MethodHookParam param) throws Throwable {
                    super.beforeHookedMethod(param);
                    Log.d(TAG, "call MainActivity->stringFromJNI arg=" + param.args[0]);
                }

                @Override
                protected void afterHookedMethod(MethodHookParam param) throws Throwable {
                    super.afterHookedMethod(param);
                    Log.d(TAG, "call stringFromJNI return:" + param.getResult());
                    param.setResult("byebye");
                }
            });

            // android.app.Activity
            XSHelpers.findAndHookMethod(Activity.class, "onCreate", Bundle.class, new XC_MethodHook(){
                @Override
                protected void beforeHookedMethod(MethodHookParam param) throws Throwable {
                    super.beforeHookedMethod(param);
                    Log.d(TAG, "call MainActivity->onCreate:" + param.args[0]);
                }
            });

            Log.d(TAG, "exit hookTestApp");
        } catch (Exception e) {
            Log.d(TAG, "hookTestApp failed:\n" + Log.getStackTraceString(e));
        }
    }

    private static void hookDeviceInfoApp(ClassLoader loader){
        //com.lucky.info.main.MainActivity
        try {
            Class<?> DeviceInfoApp = loader.loadClass("com.liuzh.deviceinfo.DeviceInfoApp");
            XSHelpers.findAndHookMethod(DeviceInfoApp, "onCreate", new XC_MethodHook(){
                @Override
                protected void beforeHookedMethod(MethodHookParam param) throws Throwable {
                    super.beforeHookedMethod(param);
                    Log.d(TAG, "call DeviceInfoApp->onCreate:");
                }
            });
            //com.liuzh.deviceinfo.splash.SplashActivity  onCreate(Bundle bundle0) {
            Class<?> SplashActivity = loader.loadClass("com.liuzh.deviceinfo.splash.SplashActivity");
            XSHelpers.findAndHookMethod(SplashActivity, "onCreate", Bundle.class, new XC_MethodHook(){
                @Override
                protected void beforeHookedMethod(MethodHookParam param) throws Throwable {
                    super.beforeHookedMethod(param);
                    Log.d(TAG, "call SplashActivity->onCreate:");
                }
            });


            //androidx.appcompat.app.AppCompatActivity.setContentView(int) : void
            Class<?> AppCompatActivity = loader.loadClass("androidx.appcompat.app.AppCompatActivity");
            XSHelpers.findAndHookMethod(AppCompatActivity, "setContentView", int.class, new XC_MethodHook(){
                @Override
                protected void beforeHookedMethod(MethodHookParam param) throws Throwable {
                    super.beforeHookedMethod(param);
                    Log.d(TAG, "call AppCompatActivity->setContentView:" + param.args[0]);
                }
            });

        }catch (Exception ex){
            Log.d(TAG, "hookDeviceInfoApp failed:\n" + Log.getStackTraceString(ex));
        }
    }

}
