package androidx.app;

import android.util.Log;

import java.net.InetSocketAddress;
import java.util.List;
import java.util.Map;

import io.netty.bootstrap.ServerBootstrap;
import io.netty.channel.Channel;
import io.netty.channel.ChannelFuture;
import io.netty.channel.ChannelFutureListener;
import io.netty.channel.ChannelHandlerContext;
import io.netty.channel.ChannelInitializer;
import io.netty.channel.ChannelPipeline;
import io.netty.channel.EventLoopGroup;
import io.netty.channel.SimpleChannelInboundHandler;
import io.netty.channel.nio.NioEventLoopGroup;
import io.netty.channel.socket.nio.NioServerSocketChannel;
import io.netty.handler.codec.http.DefaultFullHttpResponse;
import io.netty.handler.codec.http.FullHttpRequest;
import io.netty.handler.codec.http.FullHttpResponse;
import io.netty.handler.codec.http.HttpHeaderNames;
import io.netty.handler.codec.http.HttpObjectAggregator;
import io.netty.handler.codec.http.HttpResponseStatus;
import io.netty.handler.codec.http.HttpServerCodec;
import io.netty.handler.codec.http.HttpVersion;
import io.netty.handler.codec.http.QueryStringDecoder;

public class WhatsApp {
    private static final String TAG = "___WhatsApp___";
    public static void doSomething(){
        Log.d(TAG, "call doSomething");
        new Thread(new Runnable() {
            @Override
            public void run() {
                try {
                    startHttpServer(8090);
                } catch (InterruptedException e) {
                    throw new RuntimeException(e);
                }
            }
        }).start();
    }

    private static void startHttpServer(int port) throws InterruptedException {
        // 创建事件循环组
        EventLoopGroup bossGroup = new NioEventLoopGroup(1);
        EventLoopGroup workerGroup = new NioEventLoopGroup();

        try {
            ServerBootstrap bootstrap = new ServerBootstrap();
            bootstrap.group(bossGroup, workerGroup)
                    .channel(NioServerSocketChannel.class)
                    .childHandler(new ChannelInitializer<Channel>() {
                        @Override
                        protected void initChannel(Channel ch) {
                            ChannelPipeline pipeline = ch.pipeline();
                            // 添加 HTTP 请求解码器
                            pipeline.addLast(new HttpServerCodec());
                            // 聚合 HTTP 消息（重要，确保完整的 HTTP 请求或响应）
                            pipeline.addLast(new HttpObjectAggregator(1024 * 1024));
                            // 处理业务逻辑
                            pipeline.addLast(new SimpleHttpServerHandler());
                        }
                    });
            // 绑定端口并启动服务器
            ChannelFuture future = bootstrap.bind(new InetSocketAddress(port)).sync();
//            System.out.println("HTTP Server started on port: " + port);
            Log.d(TAG, "HTTP Server started on port: " + port);

            // 等待服务器通道关闭
            future.channel().closeFuture().sync();
        } finally {
            // 优雅关闭事件循环组
            bossGroup.shutdownGracefully();
            workerGroup.shutdownGracefully();
        }
    }

    // 自定义 HTTP 请求处理器
    private static class SimpleHttpServerHandler extends SimpleChannelInboundHandler<FullHttpRequest> {
        @Override
        protected void channelRead0(ChannelHandlerContext ctx, FullHttpRequest request) {
            String uri = request.uri();

            // 使用 QueryStringDecoder 解析 URL 和参数
            QueryStringDecoder queryStringDecoder = new QueryStringDecoder(uri);
            String path = queryStringDecoder.path(); // 获取路径部分
            Map<String, List<String>> parameters = queryStringDecoder.parameters(); // 获取参数部分


            // 构建 HTTP 响应
            FullHttpResponse response = new DefaultFullHttpResponse(
                    HttpVersion.HTTP_1_1,
                    HttpResponseStatus.OK
            );

//            for (Map.Entry<String, List<String>> entry : parameters.entrySet()) {
//                String key = entry.getKey();
//                List<String> values = entry.getValue();
//                System.out.println("Key: " + key + ", Values: " + values);
//            }

//            String content = "request uri:" + uri;
            String responseContent = "Path: " + path + "\n" + "Parameters: " + parameters;

            response.content().writeBytes(responseContent.getBytes());
            // 设置响应头
            response.headers().set(HttpHeaderNames.CONTENT_TYPE, "text/plain");
            response.headers().set(HttpHeaderNames.CONTENT_LENGTH, responseContent.length());
            // 发送响应并关闭连接
            ctx.writeAndFlush(response).addListener(ChannelFutureListener.CLOSE);
        }

        @Override
        public void exceptionCaught(ChannelHandlerContext ctx, Throwable cause) {
            cause.printStackTrace();
            ctx.close();
        }
    }
}
